/******************************************
TITLE:          Pinball Machine
Created by      Mark Gharby
DATE:           02/17/2022
*******************************************/
/*Parts list: 3x Break-Beam Sensors, 3x Push Buttons(Bat, Timer, Reset), Solenoid, LCD Display, LED Display
 
 Goals: 1) Have Break-Beam sensors connected to LCD display and programmed to count the "Score" up 
            BUT code sensor so there's a delay to let player reach into net to retrieve ball without
            counting score up
        2) Right-side Push Button actuates solenoid
        3) Left-side1 Push Button connected to LED Display that starts a countdown timer for 60 seconds to 0 Seconds
        4) Left-side2 Push Button connected to both LED and LCD display a4nd returns both to inital conditions 
              ie. LCD returns score to 0 and LED returns timer to 60 seconds

Current issues in the code:
1) Timer code only executes when button is held down continously - stops other code from being executed
*/

#include "SevSeg.h"
#include "Wire.h" 
#include "LiquidCrystal_I2C.h"
LiquidCrystal_I2C lcd(0x27,16,2);

#define SOLENOID 53
#define FIRSTBASE 14
#define SECONDBASE 15
#define THIRDBASE 16
#define TIMERBUTTON 17
#define BATBUTTON 18
#define RESETBUTTON 19

int BREAK=0;
int SCORE=0;

bool BATSTATE;
bool RESETSTATE;
bool TIMERSTATE;

bool FIRSTSTATE;
bool SECONDSTATE;
bool THIRDSTATE;

//--------------------LED Countdown Initial Condition--------------------//
#define ontime 700

#define A 2
#define B 3
#define C 4
#define D 5
#define E 6
#define F 7
#define G 8
#define DP 9
#define CC1 10
#define CC2 11
#define CC3 12
#define CC4 13

#define numbersegments {\
  {1,1,1,1,1,1,0,0},\ 
  {0,1,1,0,0,0,0,0},\ 
  {1,1,0,1,1,0,1,0},\
  {1,1,1,1,0,0,1,0},\ 
  {0,1,1,0,0,1,1,0},\ 
  {1,0,1,1,0,1,1,0},\ 
  {0,0,1,1,1,1,1,0},\ 
  {1,1,1,0,0,0,0,0},\ 
  {1,1,1,1,1,1,1,0},\ 
  {1,1,1,0,0,1,1,0} \ 
}
  
byte numbers[10][8] = numbersegments;
const int segments[8] = { A, B, C, D, E, F, G, DP };

void getDigitVal();  
unsigned int count=60;
int endMillis=0;
unsigned long previousTime=0;
bool starttime=0;



//--------------------set up function--------------------//
void setup() {
  // put your setup code here, to run once:
Serial.begin(9600);

//--------Outputs:--------//
 //Solenoid
      pinMode(SOLENOID, OUTPUT);
      digitalWrite(SOLENOID, LOW);
 //LED Display
  pinMode(A, OUTPUT);
  digitalWrite(A,LOW);
  pinMode(B, OUTPUT);
  digitalWrite(B,LOW);
  pinMode(C, OUTPUT);
  digitalWrite(C,LOW);
  pinMode(D, OUTPUT);
  digitalWrite(D,LOW);
  pinMode(E, OUTPUT);
  digitalWrite(E,LOW);
  pinMode(F, OUTPUT);
  digitalWrite(F,LOW);
  pinMode(G, OUTPUT);
  digitalWrite(G,LOW);
  pinMode(DP, OUTPUT);
  digitalWrite(DP,LOW);

  pinMode(CC1, OUTPUT);
  digitalWrite(CC1,HIGH);
  pinMode(CC2, OUTPUT);
  digitalWrite(CC2,HIGH);
  pinMode(CC3, OUTPUT);
  digitalWrite(CC3,HIGH);
  pinMode(CC4, OUTPUT);
  digitalWrite(CC4,HIGH);


 //LCD Display
  lcd.init();
  lcd.backlight();
  lcd.print("USF Bulls");
  lcd.setCursor(0,1);
  lcd.print("Score: ");
  
//--------Inputs:--------//
  //Buttons
      pinMode(TIMERBUTTON,INPUT_PULLUP);
      
      pinMode(BATBUTTON, INPUT_PULLUP);
      
      pinMode(RESETBUTTON,INPUT_PULLUP);
      
  //Break-Beam Sensors
      pinMode(FIRSTBASE, INPUT_PULLUP);
      
      pinMode(SECONDBASE,INPUT_PULLUP);
      
      pinMode(THIRDBASE,INPUT_PULLUP);

}

//-------------------main Arduino loop---------------------//

void loop() {
  // put your main code here, to run repeatedly:
          
// 3X PUSH BUTTONS//
TIMERSTATE=digitalRead(TIMERBUTTON);
 if (TIMERSTATE == LOW){
    Serial.println("---------------");
    Serial.println("timerbutton");
    starttime=1;
    delay(1000);
  }
 else{
    //nothing
  }

if (starttime == 1){
        endMillis=1000;
        Serial.println("----------------");
        Serial.println("Startime=1");
        if(millis()-previousTime >= endMillis){
            Serial.println("Entered Countdown loop");
            unsigned digit4 = 1U;
            unsigned digit3 = 10U;
            int ones = getDigitVal(digit4,count);
            int tens = getDigitVal(digit3,count);
            count=count-1;
                Serial.print("TIME  ");
                Serial.print(tens);
                Serial.println(ones); 
                Serial.print("PreviousTime    ");
                Serial.println(previousTime); 
                Serial.print("CurrentTime     ");
                Serial.println(millis());
                Serial.print("COUNT           ");
                Serial.println(count);
                setsegments(ones,CC4,ontime);
                setsegments(tens,CC3,ontime);
                previousTime=millis();
                  if (count<=0){
                    Serial.print("Entered last if statement       ");
                    Serial.println(count);                    
                    count=60;
                    starttime=0;
                    }
                  else{
                    //nothing
                  }
          }
          else{
            //nothing
            
          }
  }
else{
       //nothing    
     }


BATSTATE=digitalRead(BATBUTTON); //Actuate Solenoid when push button low
  if (BATSTATE == LOW){
       Serial.println("---------------");
       Serial.println("Batbutton");

   
   //SOLENOID
    digitalWrite(SOLENOID, HIGH);
    delay(30);
    digitalWrite(SOLENOID, LOW);
    delay(30);
  }
  else {
      digitalWrite(SOLENOID, LOW);
      delay(30);
    }
 
RESETSTATE=digitalRead(RESETBUTTON); //Reser timer and score and push button low
  //LCD back to Zero, Timer back to 60sec
  if (RESETSTATE == LOW){
        Serial.println("---------------");
        Serial.println("resetbutton"); 

    starttime=0;
    //Reset connected to LCD, return score to zero, and reset LED timer
    SCORE=0;
    count=60;
    starttime=0;
    BREAK=0;
    
    lcd.setCursor(8,1);
    lcd.print("   ");

    delay(600);
  }

// 3X BREAK BEAM SENSORS//
FIRSTSTATE = digitalRead(FIRSTBASE); //Readout break beam sensor
  if (FIRSTSTATE == LOW){
      Serial.println("---------------");
      Serial.println("firstbase"); 
   
        BREAK=BREAK+1; //Count up score after second break because the user's hand must go in to grab the ball
 

        if (BREAK %2 == 0){
        Serial.println("even");
        SCORE=SCORE+1;
        BREAK=0;

      
        lcd.setCursor(8,1);
        lcd.print("  ");
        lcd.setCursor(8,1);
        lcd.print(SCORE);
  
         }
 
        else {
        Serial.println("odd");
        }
   
   
    delay(900);
   
  }
    

SECONDSTATE = digitalRead(SECONDBASE);  //Readout break beam sensor
  if (SECONDSTATE == LOW){
    Serial.println("---------------");
    Serial.println("secondbase"); 

        BREAK=BREAK+1; //Count up score after second break because the user's hand must go in to grab the ball

        if (BREAK %2 == 0){
        Serial.println("even");
        SCORE=SCORE+1;
        BREAK=0;
        
        lcd.setCursor(8,1);
        lcd.print("  ");
        lcd.setCursor(8,1);
        lcd.print(SCORE);
  
        }
 
        else {
        Serial.println("odd");
         }
    
      delay(850);
    
  }

THIRDSTATE = digitalRead(THIRDBASE);  //Readout break beam sensor
  if (THIRDSTATE == LOW){
        Serial.println("---------------");
        Serial.println("thirdbase");

        BREAK=BREAK+1; //Count up score after second break because the user's hand must go in to grab the ball

        if (BREAK %2 == 0){
        Serial.println("even");
        SCORE=SCORE+1;
        BREAK=0;
        
        lcd.setCursor(8,1);
        lcd.print("  ");
        lcd.setCursor(8,1);
        lcd.print(SCORE);
          }
 
        else {
        Serial.println("odd");
        }
   
    delay(1100);
   }
 

}
